package cn.com.infosec.crypto.engines;

import cn.com.infosec.crypto.AsymmetricBlockCipher;
import cn.com.infosec.crypto.CipherParameters;
import cn.com.infosec.crypto.DataLengthException;
import cn.com.infosec.crypto.params.ElGamalKeyParameters;
import cn.com.infosec.crypto.params.ElGamalPrivateKeyParameters;
import cn.com.infosec.crypto.params.ElGamalPublicKeyParameters;
import cn.com.infosec.crypto.params.ParametersWithRandom;
import cn.com.infosec.util.BigIntegers;
import java.math.BigInteger;
import java.security.SecureRandom;

public class ElGamalEngine implements AsymmetricBlockCipher {
   private static final BigInteger ZERO = BigInteger.valueOf(0L);
   private static final BigInteger ONE = BigInteger.valueOf(1L);
   private static final BigInteger TWO = BigInteger.valueOf(2L);
   private ElGamalKeyParameters key;
   private SecureRandom random;
   private boolean forEncryption;
   private int bitSize;

   public void init(boolean forEncryption, CipherParameters param) {
      if (param instanceof ParametersWithRandom) {
         ParametersWithRandom param1;
         this.key = (ElGamalKeyParameters)(param1 = (ParametersWithRandom)param).getParameters();
         this.random = param1.getRandom();
      } else {
         this.key = (ElGamalKeyParameters)param;
         this.random = new SecureRandom();
      }

      this.forEncryption = forEncryption;
      this.bitSize = this.key.getParameters().getP().bitLength();
      if (forEncryption) {
         if (!(this.key instanceof ElGamalPublicKeyParameters)) {
            throw new IllegalArgumentException("ElGamalPublicKeyParameters are required for encryption.");
         }
      } else if (!(this.key instanceof ElGamalPrivateKeyParameters)) {
         throw new IllegalArgumentException("ElGamalPrivateKeyParameters are required for decryption.");
      }

   }

   public int getInputBlockSize() {
      return this.forEncryption ? (this.bitSize - 1) / 8 : (this.bitSize + 7) / 8 * 2;
   }

   public int getOutputBlockSize() {
      return this.forEncryption ? (this.bitSize + 7) / 8 * 2 : (this.bitSize - 1) / 8;
   }

   public byte[] processBlock(byte[] in, int inOff, int inLen) {
      if (this.key != null) {
         int var4;
         if (this.forEncryption) {
            var4 = (this.bitSize - 1 + 7) / 8;
         } else {
            var4 = this.getInputBlockSize();
         }

         if (inLen <= var4) {
            BigInteger var16 = this.key.getParameters().getP();
            BigInteger var6;
            int in2;
            byte[] inLen1;
            byte[] var10000;
            BigInteger var18;
            if (this.key instanceof ElGamalPrivateKeyParameters) {
               var10000 = in;
               int var21 = inOff;
               byte[] var10002 = in;
               int var10003 = inOff;
               int var10004 = inLen / 2;
               byte[] inOff2 = new byte[in2 = inLen / 2];
               inLen1 = new byte[var10004];
               System.arraycopy(var10002, var10003, inOff2, 0, in2);
               System.arraycopy(var10000, var21 + in2, inLen1, 0, in2);
               BigInteger in3;
               var18 = in3 = new BigInteger;
               ElGamalEngine var22 = this;
               in3.<init>(1, inOff2);
               var6 = new BigInteger.<init>(1, inLen1);
               ElGamalPrivateKeyParameters in4 = (ElGamalPrivateKeyParameters)var22.key;
               return BigIntegers.asUnsignedByteArray(var18.modPow(var16.subtract(ONE).subtract(in4.getX()), var16).multiply(var6).mod(var16));
            } else {
               if (inOff != 0 || inLen != in.length) {
                  var10000 = in;
                  in = new byte[inLen];
                  System.arraycopy(var10000, inOff, in, 0, inLen);
               }

               BigInteger inOff1;
               var18 = inOff1 = new BigInteger;
               var18.<init>(1, in);
               if (var18.bitLength() >= var16.bitLength()) {
                  throw new DataLengthException("input too large for ElGamal cipher.\n");
               } else {
                  ElGamalPublicKeyParameters in1 = (ElGamalPublicKeyParameters)this.key;
                  inLen = var16.bitLength();
                  BigInteger var5;
                  var5 = new BigInteger.<init>(inLen, this.random);

                  while(var5.equals(ZERO) || var5.compareTo(var16.subtract(TWO)) > 0) {
                     var5 = new BigInteger.<init>(inLen, this.random);
                  }

                  ElGamalEngine var19 = this;
                  var6 = this.key.getParameters().getG().modPow(var5, var16);
                  BigInteger var10001 = inOff1.multiply(in1.getY().modPow(var5, var16)).mod(var16);
                  byte[] var7 = var6.toByteArray();
                  in = var10001.toByteArray();
                  inOff = var19.getOutputBlockSize();
                  inLen1 = new byte[inOff];
                  int var8;
                  int var17;
                  byte[] var20;
                  if (var7.length > (var4 = inOff / 2)) {
                     var10000 = var7;
                     var20 = var7;
                     var8 = var4 - (var7.length - 1);
                     var17 = var20.length - 1;
                     System.arraycopy(var10000, 1, inLen1, var8, var17);
                  } else {
                     var10000 = var7;
                     var20 = var7;
                     var8 = var4 - var7.length;
                     var17 = var20.length;
                     System.arraycopy(var10000, 0, inLen1, var8, var17);
                  }

                  if (in.length > var4) {
                     var10000 = in;
                     var8 = inOff - (in.length - 1);
                     in2 = in.length - 1;
                     System.arraycopy(var10000, 1, inLen1, var8, in2);
                  } else {
                     var10000 = in;
                     var8 = inOff - in.length;
                     in2 = in.length;
                     System.arraycopy(var10000, 0, inLen1, var8, in2);
                  }

                  return inLen1;
               }
            }
         } else {
            throw new DataLengthException("input too large for ElGamal cipher.\n");
         }
      } else {
         throw new IllegalStateException("ElGamal engine not initialised");
      }
   }
}
